/*=========================================================================
 Authors: The GoFigure Dev. Team.
 at Megason Lab, Systems biology, Harvard Medical school, 2009
 
 Copyright (c) 2009, President and Fellows of Harvard College.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 Neither the name of the  President and Fellows of Harvard College
 nor the names of its contributors may be used to endorse or promote
 products derived from this software without specific prior written
 permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =========================================================================*/

#include <itkImage.h>
#include <itkImageFileReader.h>
#include <itkImageFileWriter.h>

#include "itkFastICAImageFilter.h"

using namespace itk;

int main( int argc, char** argv )
{
  if( argc < 5 )
    {
    std::cout <<"Requires 4 arguments:" <<std::endl;
    std::cout <<"1- Input 0" <<std::endl;
    std::cout <<"2- Input 1" <<std::endl;
    std::cout <<"3- Number Of Iterations" <<std::endl;
    std::cout <<"4- Sampling Ratio" <<std::endl;
    return EXIT_FAILURE;
    }
  const unsigned int Dimension = 3;
  typedef unsigned char PixelType;
  typedef Image< PixelType, Dimension > InputImageType;

  typedef double OutputPixelType;
  typedef Image< OutputPixelType, Dimension > OutputImageType;

  typedef ImageFileReader< InputImageType > InputImageReaderType;
  typedef ImageFileWriter< OutputImageType > OutputImageWriterType;

  typedef FastICAImageFilter< InputImageType, OutputImageType > FilterType;

  InputImageReaderType::Pointer reader0 = InputImageReaderType::New();
  reader0->SetFileName( argv[1] );
  reader0->Update();

  InputImageType::Pointer input0 = reader0->GetOutput();

  InputImageReaderType::Pointer reader1 = InputImageReaderType::New();
  reader1->SetFileName( argv[2] );
  reader1->Update();

  InputImageType::Pointer input1 = reader1->GetOutput();

//   FilterType::MatrixType w( 2, 2 );
//   w[0][0]= 0.36992;
//   w[0][1]=  -0.929063;
//   w[1][0]=  0.929063;
//   w[1][1]= 0.36992;

  FilterType::Pointer filter = FilterType::New();
  filter->SetInput( 0, input0 );
  filter->SetInput( 1, input1 );
//   filter->SetW( w );
  filter->SetNumberOfIterations( atoi( argv[3] ) );
  filter->SetSamplingRatio( atof( argv[4] ) );
  filter->Update();

  std::cout <<"*** RESULTS ***" <<std::endl;
  std::cout <<"W = " <<filter->GetW() <<std::endl;
  std::cout <<"A = " <<filter->GetA() <<std::endl;

  OutputImageWriterType::Pointer writer0 = OutputImageWriterType::New();
  writer0->SetInput( filter->GetOutput( 0 ) );
  writer0->SetFileName( "output0_iter1.mhd" );
  writer0->Update();

  OutputImageWriterType::Pointer writer1 = OutputImageWriterType::New();
  writer1->SetInput( filter->GetOutput( 1 ) );
  writer1->SetFileName( "output1_iter1.mhd" );
  writer1->Update();

  return EXIT_SUCCESS;
}
